"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = _default;

var _pluginSyntaxDynamicImport = _interopRequireDefault(require("@babel/plugin-syntax-dynamic-import"));

var _helpers = require("./helpers");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var asyncComponentImportNames = [];
var validImportSources = ["@jaredpalmer/after", "@jaredpalmer/after/asyncComponent"];

function _default(_ref) {
  var t = _ref.types;
  return {
    name: "after-async-component",
    inherits: _pluginSyntaxDynamicImport["default"],
    visitor: {
      ImportDeclaration: function ImportDeclaration(path) {
        if (t.isStringLiteral(path.node.source) && validImportSources.includes(path.node.source.value)) {
          var specifiers = path.node.specifiers;
          var asyncComponentImport = specifiers.filter(function (specifier) {
            return t.isImportDefaultSpecifier(specifier) || t.isImportSpecifier(specifier) && specifier.imported.name === "asyncComponent";
          });
          asyncComponentImport.forEach(function (asyncComponentImport) {
            asyncComponentImportNames.push(asyncComponentImport.local.name);
          });
        }
      },
      ObjectExpression: function ObjectExpression(path, _ref2) {
        var opts = _ref2.opts;
        // if there was no import statement just return
        if (asyncComponentImportNames.length === 0) return; // check for "component" in object properties and function call in value { component: hello() }

        var component = path.node.properties.find(function (property) {
          return t.isIdentifier(property.key, {
            name: "component"
          }) && t.isCallExpression(property.value);
        }); // if there was not propery to match conditions just return

        if (!component) return;
        path.traverse(callVisitor, {
          parentPath: path,
          prefix: opts.prefix,
          t: t
        });
      }
    }
  };
}

var callVisitor = {
  CallExpression: function CallExpression(path) {
    var t = this.t; // check if the function that get called is named "asyncComponent" or named export { asyncComponent as foo }

    if (t.isIdentifier(path.node.callee) && asyncComponentImportNames.includes(path.node.callee.name)) {
      path.traverse(importVisitor, {
        parentPath: path,
        prefix: this.prefix,
        t: t
      });
    }
  }
};
var importVisitor = {
  Import: function Import(path) {
    var argPath = (0, _helpers.getImportArgPath)(path);
    var node = argPath.node;
    var generatedChunkName = (0, _helpers.getMagicCommentChunkName)(node);
    var existingChunkNameFromParams = this.parentPath.node.arguments[1] && this.parentPath.node.arguments[1].value || null;
    var existingChunkName = (0, _helpers.existingMagicCommentChunkName)(node);
    var chunkName = convertChunkName(existingChunkNameFromParams || existingChunkName || generatedChunkName, this.prefix);
    (0, _helpers.addChunkNameToNode)(argPath, chunkName);
    this.parentPath.node.arguments[1] = this.t.stringLiteral(chunkName);
    this.parentPath.node.arguments[1] = generatedChunkName === "[request]" ? this.t.identifier(node.expressions[0].name) : this.t.stringLiteral(chunkName);
  }
};

function convertChunkName(chunkName) {
  var prefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "";
  if (chunkName === "[request]") return chunkName;
  return prefix + chunkName.replace("/", "-");
}